<?php

/**
 * FluxBB File Uploader v1.1.0
 * A simple plugin to upload files
 * Copyright (C) Loïc Dagallier
 * Compatible with FluxBB 1.4.3+ and 1.5
 * License: http://www.gnu.org/licenses/gpl.html GPL version 2 or higher
 */

/*******************************/
/****  START CONFIGURATION  ****/
/*******************************/

// Uploads directory
define('UPLOAD_ROOT', 'img/uploads/');

// Min and Max file sizes in bytes
define('UPLOAD_MIN_SIZE', 10);
define('UPLOAD_MAX_SIZE', 20971520);

// Max height and width of images
define('IMG_MAX_HEIGHT', 1024);
define('IMG_MAX_WIDTH', 1024);

// Uploaded files must be renamed ?
// (SHOULD ALWAYS BE 'true' FOR SECURITY REASONS)
define('RENAME_FILE', true);

// A user can delete his own files ?
define('ALLOW_USER_DELETE_OWN_FILES', true);

// A file used in a post can be deleted ?
define('ALLOW_DELETE_FILE_IN_USE', false);

// Image extenions allowed (note that *.ico is only supported by PHP > 5.3.0)
$upload_ext_img = array('jpg', 'jpeg', 'jpc', 'jp2', 'jpx', 'gif', 'png', 'bmp', 'tif', 'tiff');

// Other file extensions allowed
$upload_ext_doc = array('doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx', 'odt', 'ott', 'ods', 'ots', 'odp', 'otp', 'zip', 'rar', '7z', 'pdf', 'txt', 'ini');

/*****************************/
/****  END CONFIGURATION  ****/
/*****************************/

// FluxBB
define('PUN_ROOT', dirname(__FILE__).'/');
require PUN_ROOT.'include/common.php';

// Language
if (file_exists(PUN_ROOT.'lang/'.$pun_user['language'].'/upload.php'))
	require PUN_ROOT.'lang/'.$pun_user['language'].'/upload.php';
else
	require PUN_ROOT.'lang/English/upload.php';

// Guest ?
if ($pun_user['is_guest'])
	message($lang_upload['Guest'], false, '403 Forbidden');

// Install file ?
if (file_exists(PUN_ROOT.'upload_install.php'))
	message($lang_upload['ErrInstallExists'], false);

// File upload is enabled in PHP ?
if (!ini_get('file_uploads'))
	message($lang_upload['ErrConfigPHP'], false);

// Check "Uploads" directory
if (!file_exists(PUN_ROOT.UPLOAD_ROOT) || !is_writable(PUN_ROOT.UPLOAD_ROOT))
	message($lang_upload['ErrUploadRoot'], false);

// Action to do
$actions = array('add', 'manage', 'delete', 'bbcode', 'upload');

if (!isset($_REQUEST['action']) || !in_array($_REQUEST['action'], $actions))
	$_REQUEST['action'] = 'add';

// A file was uploaded
if ($_REQUEST['action'] == 'upload' && isset($_FILES['userfile']))
{
	// Errors
	switch ($_FILES['userfile']['error'])
	{
		case 1: // UPLOAD_ERR_INI_SIZE
			message($lang_upload['ErrSizeMaxPHP'], false);
		case 2: // UPLOAD_ERR_FORM_SIZE
			message($lang_upload['ErrSizeMaxAdmin'], false);
		case 3: // UPLOAD_ERR_NO_FILE
			message($lang_upload['ErrNoFile'], false);
		case 4: // UPLOAD_ERR_PARTIAL
			message($lang_upload['ErrFilePartial'], false);
		case 6: // UPLOAD_ERR_NO_TMP_DIR
			message($lang_upload['ErrNoTmpDir'], false);
		case 7: // UPLOAD_ERR_CANT_WRITE
			message($lang_upload['ErrCantWrite'], false);
		case 8: // UPLOAD_ERR_EXTENSION
			message($lang_upload['ErrExtension'], false);
	}

	// Get file info
	$upload_file_time = time();
	$upload_file_name = basename($_FILES['userfile']['name']);
	$upload_file_tmp = $_FILES['userfile']['tmp_name'];
	$upload_file_size = filesize($upload_file_tmp);
	$upload_file_ext = strtolower(substr(strrchr($upload_file_name, '.'), 1));

	// Is an uploaded file ?
	if (!is_uploaded_file($upload_file_tmp))
		message($lang_upload['ErrFileForm'], false);

	// Max size
	if ($upload_file_size > UPLOAD_MAX_SIZE)
		message($lang_upload['ErrSizeMaxAdmin'], false);

	// Min size
	if ($upload_file_size < 1 || $upload_file_size < UPLOAD_MIN_SIZE)
		message($lang_upload['ErrSizeMinAdmin'], false);

	// Filename
	$upload_file_name = preg_replace('#[\x00-\x1f]#', '', $upload_file_name);
	$upload_file_name = preg_replace('#[@=\\\'/ ]+#', '_', $upload_file_name);

	// Type (by extension)
	if (!in_array($upload_file_ext, $upload_ext_img) && !in_array($upload_file_ext, $upload_ext_doc))
		message($lang_upload['ErrFileNotAllowed'], false);

	// If the file in an image
	if (in_array($upload_file_ext, $upload_ext_img))
	{
		$upload_file_dim = getimagesize($upload_file_tmp);

		if (!$upload_file_dim || empty($upload_file_dim[0]) || empty($upload_file_dim[1]))
			message($lang_upload['ErrUploadFailed'], true);

		if ($upload_file_dim[0] > IMG_MAX_WIDTH || $upload_file_dim[1] > IMG_MAX_HEIGHT)
			message(sprintf($lang_upload['ErrImgSize'], forum_number_format(IMG_MAX_WIDTH), forum_number_format(IMG_MAX_HEIGHT)), false);
	}

	// Rename file ?
	if (RENAME_FILE)
	{
		do
		{
			$upload_file_dest = md5(uniqid(mt_rand().$upload_file_time, true)).'.'.$upload_file_ext;
		}
		while (file_exists(PUN_ROOT.UPLOAD_ROOT.$upload_file_dest));
	}
	else
	{
		$upload_file_dest = $upload_file_name;

		if (file_exists(PUN_ROOT.UPLOAD_ROOT.$upload_file_dest))
			message($lang_upload['ErrFileExists'], false);
	}

	// Copy file
	if(!move_uploaded_file($upload_file_tmp, PUN_ROOT.UPLOAD_ROOT.$upload_file_dest))
		message($lang_upload['ErrUploadFailed'], false);

	// Change chmod
	@chmod(PUN_ROOT.UPLOAD_ROOT.$upload_file_dest, 0644);

	// Update database
	$db->query('INSERT INTO '.$db->prefix.'uploads (up_user, up_init, up_dest, up_time) VALUES('.$pun_user['id'].', \''.$db->escape($upload_file_name).'\', \''.$db->escape($upload_file_dest).'\', '.$upload_file_time.')') or error('Unable to add upload', __FILE__, __LINE__, $db->error());

	// Create BBCode
	if (in_array($upload_file_ext, $upload_ext_img))
		$upload_file_bbcode = '[img='.pun_htmlspecialchars($upload_file_name).']'.get_base_url().'/'.UPLOAD_ROOT.pun_htmlspecialchars($upload_file_dest).'[/img]';
	else
		$upload_file_bbcode = '[url='.get_base_url().'/'.UPLOAD_ROOT.pun_htmlspecialchars($upload_file_dest).']'.pun_htmlspecialchars($upload_file_name).'[/url]';

	// Successfully uploaded !
	message(sprintf($lang_upload['MsgSuccessUpload'], $upload_file_bbcode), true);
}

// Display BBCode of an existing file
if ($_REQUEST['action'] == 'bbcode')
{
	$_GET['file'] = intval($_GET['file']);

	$result = $db->query('SELECT up_init, up_dest FROM '.$db->prefix.'uploads WHERE up_id='.$db->escape($_GET['file'])) or error('Unable to fetch file list', __FILE__, __LINE__, $db->error());

	if (!$db->num_rows($result))
		message($lang_upload['ErrFileNotExist'], false);

	$cur_file = $db->fetch_assoc($result);
	$cur_file['up_ext'] = strtolower(substr(strrchr($cur_file['up_dest'], '.'), 1));

	if (in_array($cur_file['up_ext'], $upload_ext_img))
		$cur_file['up_bbcode'] = '[img='.pun_htmlspecialchars($cur_file['up_init']).']'.get_base_url().'/'.UPLOAD_ROOT.pun_htmlspecialchars($cur_file['up_dest']).'[/img]';
	else
		$cur_file['up_bbcode'] = '[url='.get_base_url().'/'.UPLOAD_ROOT.pun_htmlspecialchars($cur_file['up_dest']).']'.pun_htmlspecialchars($cur_file['up_init']).'[/url]';

	message(sprintf($lang_upload['MsgBBCode'], $cur_file['up_bbcode']), false);
}

// Delete file
if ($_REQUEST['action'] == 'delete' && isset($_POST['file']))
{
	if (!ALLOW_USER_DELETE_OWN_FILES)
		message($lang_upload['ErrUserCantDelete'], true);

	$_POST['file'] = intval($_POST['file']);

	$result = $db->query('SELECT * FROM '.$db->prefix.'uploads WHERE up_id='.$db->escape($_POST['file'])) or error('Unable to fetch file info', __FILE__, __LINE__, $db->error());

	if (!$db->num_rows($result))
		message($lang_upload['ErrFileNotExist'], true);

	$cur_file = $db->fetch_assoc($result);

	// Are you the owner of the file ?
	if ($cur_file['up_user'] != $pun_user['id'])
		message($lang_upload['ErrUserCantDelete'], true);

	// If false, the file mustn't be in use
	if (!ALLOW_DELETE_FILE_IN_USE)
	{
		$result = $db->query('SELECT * FROM '.$db->prefix.'posts WHERE message LIKE \'%'.$cur_file['up_dest'].'%\'') or error('Unable to check if the file is used', __FILE__, __LINE__, $db->error());

		if ($db->num_rows($result) > 0)
			message($lang_upload['ErrFileIsUsed'], true);
	}

	// Clear database and delete file in directory
	$db->query('DELETE FROM '.$db->prefix.'uploads WHERE up_id='.$cur_file['up_id']) or error('Unable to delete file', __FILE__, __LINE__, $db->error());
	@unlink(PUN_ROOT.UPLOAD_ROOT.$cur_file['up_dest']);

	// Successfully deleted !
	redirect('upload.php?action=manage', $lang_upload['MsgSuccessDelete']);
}

// Display a page
$page_title = array(pun_htmlspecialchars($pun_config['o_board_title']), $lang_upload['Title']);
define('PUN_ACTIVE_PAGE', 'uploads');
require PUN_ROOT.'header.php';

?>

<div id="uploads" class="block2col">

	<div class="blockmenu">
		<h2><span><?php echo $lang_upload['MenuFile'] ?></span></h2>
		<div class="box">
			<div class="inbox">
				<ul>
					<li<?php if ($_REQUEST['action'] == 'add') echo ' class="isactive"'; ?>><a href="upload.php?action=add"><?php echo $lang_upload['MenuAdd'] ?></a></li>
					<li<?php if ($_REQUEST['action'] == 'manage') echo ' class="isactive"'; ?>><a href="upload.php?action=manage"><?php echo $lang_upload['MenuManage'] ?></a></li>
				</ul>
			</div>
		</div>
	</div>

<?php

if ($_REQUEST['action'] == 'manage')
{

?>
	<div class="block">
		<div class="blocktable">
			<h2><span><?php echo $lang_upload['MenuManage']?></span></h2>
			<div class="box">
				<div class="inbox">
					<table class="aligntop" cellspacing="0">
					<thead>
						<tr>
							<th class="tcl" scope="col" style="width:45%"><?php echo $lang_upload['MyFiles'] ?></th>
							<th class="tc2" scope="col" style="width:15%"><?php echo $lang_upload['Size'] ?></th>
							<th class="tc2" scope="col" style="width:20%"><?php echo $lang_upload['Added'] ?></th>
							<th class="tc2" scope="col" style="width:20%"><?php echo $lang_upload['Actions'] ?></th>
						</tr>
					</thead>
					<tbody>

<?php

	$result = $db->query('SELECT up_id, up_user, up_init, up_dest, up_time FROM '.$db->prefix.'uploads WHERE up_user='.$pun_user['id']) or error('Unable to fetch file list', __FILE__, __LINE__, $db->error());

	while ($cur_file = $db->fetch_assoc($result))
	{
		$cur_file['up_time'] = format_time($cur_file['up_time']);
		$cur_file['up_size'] = forum_number_format(filesize(UPLOAD_ROOT.$cur_file['up_dest']));

		if (!ALLOW_USER_DELETE_OWN_FILES)
			$cur_file['up_delete'] = null;
		else
			$cur_file['up_delete'] = '<a href="upload.php?action=delete&amp;file='.$cur_file['up_id'].'">'.$lang_upload['Delete'].'</a> - ';

?>

					<tr class="rowodd">
						<td class="tcl"><div class="tclcon"><?php echo $cur_file['up_init'] ?></div></td>
						<td class="tc2"><?php echo $cur_file['up_size'] ?></td>
						<td class="tc2"><?php echo $cur_file['up_time'] ?></td>
						<td class="tc2">
							<?php echo $cur_file['up_delete'] ?>
							<a href="upload.php?action=bbcode&amp;file=<?php echo $cur_file['up_id'] ?>">BBCode</a>
						</td>
					</tr>

<?php

	}

?>

					</tbody>
					</table>
				</div>
			</div>
		</div>
	</div>

</div>

<?php

	require PUN_ROOT.'footer.php';
}

if ($_REQUEST['action'] == 'delete' && isset($_GET['file']))
{
	$_GET['file'] = intval($_GET['file']);

?>

	<div class="blockform">
		<h2><span><?php echo $lang_upload['DeleteFile']?></span></h2>
		<div class="box">
			<form method="post" action="upload.php">
				<input type="hidden" name="action" value="delete" />
				<input type="hidden" name="file" value="<?php echo $_GET['file'] ?>" />
				<input type="hidden" name="delete_multiple_comply" value="1" />
				<div class="inform">
					<div class="forminfo">
						<p><?php echo $lang_upload['MsgDeleteComply'] ?></p>
					</div>
				</div>
			<p class="buttons"><input type="submit" name="delete" value="<?php echo $lang_upload['Delete'] ?>" /> <a href="javascript:history.go(-1)"><?php echo $lang_common['Go back'] ?></a></p>
		</form>
	</div>

</div>

<?php

	require PUN_ROOT.'footer.php';
}

?>

	<div class="blockform">
		<h2><span><?php echo $lang_upload['AddFile']?></span></h2>
		<div class="box">
			<form id="uploadfile" action="upload.php?action=upload" enctype="multipart/form-data" method="post">
				<div class="inform">
					<fieldset>
						<legend><?php echo $lang_upload['SelectFile'] ?></legend>
						<div class="infldset">
							<label class="conl">
								<input type="hidden" name="MAX_FILE_SIZE" value="<?php echo UPLOAD_MAX_SIZE; ?>">
								<input type="file" name="userfile" />
							</label>
							<p class="buttons">
								<input type="submit" value="<?php echo $lang_upload['Upload']; ?>" />
							</p>
						</div>
					</fieldset>
				</div>
			</form>
		</div>

	</div>

</div>

<?php

require PUN_ROOT.'footer.php';

?>
